import 'package:flutter/widgets.dart';
import 'package:i18next/i18next.dart';
import 'package:i18nu/i18nu.dart';
import 'package:intl/intl.dart';

class MyL10nDelegate extends L10nDelegate {
  const MyL10nDelegate({
    List<Locale> supported,
    @required this.dataSource,
  })  : assert(dataSource != null),
        super(supported: supported);

  final AssetBundleL10nDataSource dataSource;

  @override
  Future<L10n> loadL10n(Locale locale) async {
    final namespaces = await dataSource.load(locale);
    return _I18NextWrapper(locale, namespaces);
  }
}

class _I18NextWrapper extends I18Next implements L10n {
  _I18NextWrapper(Locale locale, Map<String, Map<String, Object>> namespaces)
      : super(locale, (namespace, locale) => namespaces[namespace],
            formatter: customFormatter);

  @override
  String localize(
    String key, {
    String context,
    int count,
    Map<String, Object> arguments,
  }) =>
      t(key, context: context, count: count, arguments: arguments);

  static String customFormatter(Object value, String format, Locale locale) {
    switch (format) {
      case 'uppercase':
        return value.toString().toUpperCase();
      case 'lowercase':
        return value.toString().toLowerCase();
      default:
        if (value is DateTime) {
          return DateFormat(format, locale?.toString()).format(value);
        }
    }
    return value.toString();
  }
}
