import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:i18nu/i18nu.dart';
import 'package:mockito/mockito.dart';

class MockL10n extends Mock implements L10n {}

class MockLocalizationDelegate extends LocalizationsDelegate<L10n> {
  MockLocalizationDelegate(this.l10n);

  final L10n l10n;

  @override
  bool isSupported(Locale locale) => true;

  @override
  bool shouldReload(LocalizationsDelegate<L10n> old) => true;

  @override
  Future<L10n> load(Locale locale) async => l10n;
}

void main() {
  group('.of', () {
    BuildContext capturedContext;

    final builder = Builder(
      builder: (context) {
        capturedContext = context;
        return Container();
      },
    );

    setUp(() {
      capturedContext = null;
    });

    testWidgets('when not in hierarchy', (tester) async {
      await tester.pumpWidget(builder);
      expect(L10n.of(capturedContext), isNull);
    });

    testWidgets('when in hierarchy', (tester) async {
      const locale = Locale('en');
      final l10n = MockL10n();

      await tester.pumpWidget(Localizations(
        locale: locale,
        delegates: [
          DefaultWidgetsLocalizations.delegate,
          MockLocalizationDelegate(l10n),
        ],
        child: builder,
      ));
      await tester.pump();

      expect(L10n.of(capturedContext), equals(l10n));
    });
  });
}
